import os
import shutil
import tarfile
import tempfile

import pytest


@pytest.fixture(scope='session')
def mytest_keys():
    return ['foo', 'bar/baz']


@pytest.fixture(scope='session')
def mytest_folder(mytest_keys):
    d = tempfile.mkdtemp()
    for key in mytest_keys:
        dirname, filename = os.path.split(key)
        if dirname:
            os.makedirs(os.path.join(d, dirname))
        with open(os.path.join(d, key), 'w') as outfile:
            outfile.write('test')
    yield d
    shutil.rmtree(d)


@pytest.fixture(scope='session')
def mytest_tarfile(mytest_folder):
    filehandle, filename = tempfile.mkstemp(suffix='.tar')
    os.close(filehandle)
    os.remove(filename)
    tf = tarfile.open(filename, 'w')
    tf.add(mytest_folder, arcname='')
    tf.close()
    yield filename
    os.remove(filename)



from __future__ import absolute_import

import os.path

from .loaders.folder import FolderLoader
from .loaders.tarfile import TarfileLoader


def open(path):
    path = os.path.realpath(path)
    if os.path.isfile(path):
        return TarfileLoader(path)
    elif os.path.isdir(path):
        return FolderLoader(path)
    else:
        raise ValueError("Couldn't infer type of \"%s\"" % path)



from pencroft import TarfileLoader


def test_keys(mytest_tarfile, mytest_keys):
    loader = TarfileLoader(mytest_tarfile)
    assert sorted(mytest_keys) == sorted(loader.keys())


def test_get(mytest_tarfile):
    loader = TarfileLoader(mytest_tarfile)
    keys = loader.keys()
    for key in keys:
        loader.get(key)



import os


class FolderLoader(object):
    def __init__(self, path):
        path = os.path.realpath(path)
        self.root = path

    def _in_root(self, path):
        if os.path.isabs(path):
            if not os.path.commonprefix(
                    [path, self.root]).startswith(self.root):
                raise ValueError('"%s" is not in "%s"' % (path, self.root))
        else:
            path = os.path.join(self.root, path)
        return path

    def keys(self):
        result = []
        for dirpath, dirnames, filenames in os.walk(self.root):
            for filename in filenames:
                result.append(os.path.relpath(
                    os.path.join(dirpath, filename),
                    self.root,
                ))
        return result

    def exists(self, key):
        path = self._in_root(key)
        return os.path.exists(path)

    def get(self, key):
        path = self._in_root(key)
        with open(path, 'rb') as infile:
            return infile.read()



from __future__ import absolute_import

import argparse

from . import benchmark


def main():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()

    benchmark.set_parser(subparsers.add_parser('benchmark'))

    args = parser.parse_args()
    args.func(args)



from __future__ import absolute_import

import tarfile


class TarfileLoader(object):
    """Loads from a tarfile.

    Cannot currently be used in conjunction with multiprocessing.
    To do it, we'd have to re-open the file in each process.
    """
    def __init__(self, path):
        self.file = tarfile.open(path)
        self._names_to_members = None

    def _set_names_to_members(self):
        assert self._names_to_members is None
        self._names_to_members = dict(
            [(m.name, m) for m in self.file.getmembers() if m.isfile()])

    def keys(self):
        if self._names_to_members is None:
            self._set_names_to_members()
        return list(self._names_to_members.keys())

    def exists(self, key):
        if self._names_to_members is None:
            self._set_names_to_members()
        return key in self._names_to_members

    def get(self, key):
        member = self._names_to_members[key]
        return self.file.extractfile(member).read()



from __future__ import (
    absolute_import,
    print_function,
)

import functools
import random
import time

from . import open as open_path


def set_parser(parser):
    parser.add_argument('path')
    parser.add_argument('-t', '--threads', type=int, default=1)
    parser.add_argument('-l', '--thread-library', default='threading',
                        choices=['threading', 'multiprocessing'])
    parser.set_defaults(func=main)


class _Timer(object):
    def __init__(self, message):
        self.message = message

    def __enter__(self):
        self.start = time.time()

    def __exit__(self, type, value, traceback):
        print('%20s: %f' % (self.message, time.time() - self.start))


def get_data(loader, key):
    data = loader.get(key)
    return len(data)


def main(args):
    path = args.path
    print('%20s: %s' % ('File', path))
    print('%20s: %d' % ('Threads', args.threads))
    if args.threads > 1:
        print('%20s: %s' % ('Library', args.thread_library))

    if args.threads > 1:
        import multiprocessing.pool
        if args.thread_library == 'threading':
            pool = multiprocessing.pool.ThreadPool(args.threads)
        elif args.thread_library == 'multiprocessing':
            pool = multiprocessing.pool.Pool(args.threads)

    with _Timer('Total'):
        with _Timer('Initialization'):
            loader = open_path(path)

        with _Timer('Read keys'):
            keys = loader.keys()

        random.shuffle(keys)

        with _Timer('Read data'):
            if args.threads > 1:
                pool.map(functools.partial(get_data, loader), keys)
            else:
                for key in keys:
                    loader.get(key)
    print()



import pytest

from pencroft.benchmark import main


class Args(object):
    pass


def test_folder_1(mytest_folder):
    args = Args()
    args.path = mytest_folder
    args.threads = 1
    main(args)


def test_folder_4thread(mytest_folder):
    args = Args()
    args.path = mytest_folder
    args.threads = 4
    args.thread_library = 'threading'
    main(args)


def test_folder_4proc(mytest_folder):
    args = Args()
    args.path = mytest_folder
    args.threads = 4
    args.thread_library = 'multiprocessing'
    main(args)


def test_tarfile_1(mytest_tarfile):
    args = Args()
    args.path = mytest_tarfile
    args.threads = 1
    main(args)


def test_tarfile_4thread(mytest_tarfile):
    args = Args()
    args.path = mytest_tarfile
    args.threads = 4
    args.thread_library = 'threading'
    main(args)


@pytest.mark.xfail
def test_tarfile_4proc(mytest_tarfile):
    args = Args()
    args.path = mytest_tarfile
    args.threads = 4
    args.thread_library = 'multiprocessing'
    main(args)



import os.path
import setuptools

LOCAL_DIR = os.path.dirname(os.path.abspath(__file__))

# Get test requirements
test_requirements = []
with open(os.path.join(LOCAL_DIR, 'requirements-test.txt'), 'r') as infile:
    for line in infile:
        line = line.strip()
        if line and not line[0] == '#':  # ignore comments
            test_requirements.append(line)

setuptools.setup(
    name='pencroft',
    version='0.1',
    license='BSD',
    packages=setuptools.find_packages(),
    extras_require={'test': test_requirements},
    include_package_data=True,
    zip_safe=False,
)
